@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2024 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,12 +33,13 @@ import org.apfloat.Apint;
 import org.apfloat.ApintMath;
 import org.apfloat.Aprational;
 import org.apfloat.AprationalMath;
+import org.apfloat.FixedPrecisionApcomplexHelper;
 import org.apfloat.spi.Util;
 
 /**
  * Arbitrary precision calculator implementation.
  *
- * @version 1.11.0
+ * @version 1.13.0
  * @author Mikko Tommila
  */
 
@@ -47,7 +48,7 @@ public class ApfloatCalculatorImpl
 {
     private static final long serialVersionUID = 1L;
 
-    private static class ApcomplexFunctions
+    private class ApcomplexFunctions
         implements Functions
     {
         @Override
@@ -141,13 +142,37 @@ public class ApfloatCalculatorImpl
         @Override
         public Number acos(Number x)
         {
-            return ApcomplexMath.acos((Apcomplex) x);
+            return fixedOrArbitraryPrecision(fixedPrecisionApcomplexHelper::acos, ApcomplexMath::acos, x);
         }
 
         @Override
         public Number acosh(Number x)
         {
-            return ApcomplexMath.acosh((Apcomplex) x);
+            return fixedOrArbitraryPrecision(fixedPrecisionApcomplexHelper::acosh, ApcomplexMath::acosh, x);
+        }
+
+        @Override
+        public Number airyAi(Number x)
+        {
+            return fixedOrArbitraryPrecision(fixedPrecisionApcomplexHelper::airyAi, ApcomplexMath::airyAi, x);
+        }
+
+        @Override
+        public Number airyAiPrime(Number x)
+        {
+            return fixedOrArbitraryPrecision(fixedPrecisionApcomplexHelper::airyAiPrime, ApcomplexMath::airyAiPrime, x);
+        }
+
+        @Override
+        public Number airyBi(Number x)
+        {
+            return fixedOrArbitraryPrecision(fixedPrecisionApcomplexHelper::airyBi, ApcomplexMath::airyBi, x);
+        }
+
+        @Override
+        public Number airyBiPrime(Number x)
+        {
+            return fixedOrArbitraryPrecision(fixedPrecisionApcomplexHelper::airyBiPrime, ApcomplexMath::airyBiPrime, x);
         }
 
         @Override
@@ -185,6 +210,48 @@ public class ApfloatCalculatorImpl
             return AprationalMath.bernoulli(n);
         }
 
+        @Override
+        public Number besselI(Number x, Number y)
+        {
+            return ApcomplexMath.besselI((Apcomplex) x, (Apcomplex) y);
+        }
+
+        @Override
+        public Number besselJ(Number x, Number y)
+        {
+            return ApcomplexMath.besselJ((Apcomplex) x, (Apcomplex) y);
+        }
+
+        @Override
+        public Number besselK(Number x, Number y)
+        {
+            return ApcomplexMath.besselK((Apcomplex) x, (Apcomplex) y);
+        }
+
+        @Override
+        public Number besselY(Number x, Number y)
+        {
+            return ApcomplexMath.besselY((Apcomplex) x, (Apcomplex) y);
+        }
+
+        @Override
+        public Number beta(Number a, Number b)
+        {
+            return ApcomplexMath.beta((Apcomplex) a, (Apcomplex) b);
+        }
+
+        @Override
+        public Number beta(Number x, Number a, Number b)
+        {
+            return ApcomplexMath.beta((Apcomplex) x, (Apcomplex) a, (Apcomplex) b);
+        }
+
+        @Override
+        public Number beta(Number x1, Number x2, Number a, Number b)
+        {
+            return ApcomplexMath.beta((Apcomplex) x1, (Apcomplex) x2, (Apcomplex) a, (Apcomplex) b);
+        }
+
         @Override
         public Number binomial(Number x, Number y)
         {
@@ -220,12 +287,24 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.cos((Apcomplex) x);
         }
 
+        @Override
+        public Number cosIntegral(Number x)
+        {
+            return ApcomplexMath.cosIntegral((Apcomplex) x);
+        }
+
         @Override
         public Number cosh(Number x)
         {
             return ApcomplexMath.cosh((Apcomplex) x);
         }
 
+        @Override
+        public Number coshIntegral(Number x)
+        {
+            return ApcomplexMath.coshIntegral((Apcomplex) x);
+        }
+
         @Override
         public Number digamma(Number x)
         {
@@ -243,6 +322,36 @@ public class ApfloatCalculatorImpl
             return ApfloatMath.e(n);
         }
 
+        @Override
+        public Number ellipticE(Number x)
+        {
+            return fixedOrArbitraryPrecision(fixedPrecisionApcomplexHelper::ellipticE, ApcomplexMath::ellipticE, x);
+        }
+
+        @Override
+        public Number ellipticK(Number x)
+        {
+            return fixedOrArbitraryPrecision(fixedPrecisionApcomplexHelper::ellipticK, ApcomplexMath::ellipticK, x);
+        }
+
+        @Override
+        public Number erf(Number x)
+        {
+            return ApcomplexMath.erf((Apcomplex) x);
+        }
+
+        @Override
+        public Number erfc(Number x)
+        {
+            return ApcomplexMath.erfc((Apcomplex) x);
+        }
+
+        @Override
+        public Number erfi(Number x)
+        {
+            return ApcomplexMath.erfi((Apcomplex) x);
+        }
+
         @Override
         public Number euler(Number x)
         {
@@ -260,6 +369,18 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.exp((Apcomplex) x);
         }
 
+        @Override
+        public Number expIntegralE(Number x, Number y)
+        {
+            return ApcomplexMath.expIntegralE((Apcomplex) x, (Apcomplex) y);
+        }
+
+        @Override
+        public Number expIntegralEi(Number x)
+        {
+            return ApcomplexMath.expIntegralEi((Apcomplex) x);
+        }
+
         @Override
         public Number factorial(Number x)
         {
@@ -283,6 +404,18 @@ public class ApfloatCalculatorImpl
             throw new IllegalArgumentException("Frac can only be used with scalar values");
         }
 
+        @Override
+        public Number fresnelC(Number x)
+        {
+            return ApcomplexMath.fresnelC((Apcomplex) x);
+        }
+
+        @Override
+        public Number fresnelS(Number x)
+        {
+            return ApcomplexMath.fresnelS((Apcomplex) x);
+        }
+
         @Override
         public Number glaisher(Number x)
         {
@@ -318,18 +451,54 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.hypergeometric0F1((Apcomplex) a, (Apcomplex) z);
         }
 
+        @Override
+        public Number hypergeometric0F1Regularized(Number a, Number z)
+        {
+            return ApcomplexMath.hypergeometric0F1Regularized((Apcomplex) a, (Apcomplex) z);
+        }
+
         @Override
         public Number hypergeometric1F1(Number a, Number b, Number z)
         {
             return ApcomplexMath.hypergeometric1F1((Apcomplex) a, (Apcomplex) b, (Apcomplex) z);
         }
 
+        @Override
+        public Number hypergeometric1F1Regularized(Number a, Number b, Number z)
+        {
+            return ApcomplexMath.hypergeometric1F1Regularized((Apcomplex) a, (Apcomplex) b, (Apcomplex) z);
+        }
+
         @Override
         public Number hypergeometric2F1(Number a, Number b, Number c, Number z)
         {
             return ApcomplexMath.hypergeometric2F1((Apcomplex) a, (Apcomplex) b, (Apcomplex) c, (Apcomplex) z);
         }
 
+        @Override
+        public Number hypergeometric2F1Regularized(Number a, Number b, Number c, Number z)
+        {
+            return ApcomplexMath.hypergeometric2F1Regularized((Apcomplex) a, (Apcomplex) b, (Apcomplex) c, (Apcomplex) z);
+        }
+
+        @Override
+        public Number hypergeometricU(Number a, Number b, Number z)
+        {
+            return ApcomplexMath.hypergeometricU((Apcomplex) a, (Apcomplex) b, (Apcomplex) z);
+        }
+
+        @Override
+        public Number inverseErf(Number x)
+        {
+            throw new IllegalArgumentException("Inverse erf can only be used with scalar values");
+        }
+
+        @Override
+        public Number inverseErfc(Number x)
+        {
+            throw new IllegalArgumentException("Inverse erfc can only be used with scalar values");
+        }
+
         @Override
         public Number khinchin(Number x)
         {
@@ -359,6 +528,12 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.logGamma((Apcomplex) x);
         }
 
+        @Override
+        public Number logIntegral(Number x)
+        {
+            return ApcomplexMath.logIntegral((Apcomplex) x);
+        }
+
         @Override
         public Number max(Number x, Number y)
         {
@@ -400,6 +575,23 @@ public class ApfloatCalculatorImpl
             return ApfloatMath.pi(n);
         }
 
+        @Override
+        public Number pochhammer(Number x, Number y)
+        {
+            return ApcomplexMath.pochhammer((Apcomplex) x, (Apcomplex) y);
+        }
+
+        @Override
+        public Number polygamma(Number x, Number y)
+        {
+            if (!isLong(x))
+            {
+                throw new IllegalArgumentException("Polygamma can only be used with a valid integer first argument");
+            }
+            long n = x.longValue();
+            return ApcomplexMath.polygamma(n, (Apcomplex) y);
+        }
+
         @Override
         public Number random(Number x)
         {
@@ -458,12 +650,24 @@ public class ApfloatCalculatorImpl
             return ApcomplexMath.sin((Apcomplex) x);
         }
 
+        @Override
+        public Number sinIntegral(Number x)
+        {
+            return ApcomplexMath.sinIntegral((Apcomplex) x);
+        }
+
         @Override
         public Number sinh(Number x)
         {
             return ApcomplexMath.sinh((Apcomplex) x);
         }
 
+        @Override
+        public Number sinhIntegral(Number x)
+        {
+            return ApcomplexMath.sinhIntegral((Apcomplex) x);
+        }
+
         @Override
         public Number sqrt(Number x)
         {
@@ -668,11 +872,11 @@ public class ApfloatCalculatorImpl
 
         protected boolean isLong(Number value)
         {
-            return new Apint(value.longValue()).equals(value);
+            return value instanceof Long || value != null && new Apint(value.longValue()).equals(value);
         }
     }
 
-    private static class ApfloatFunctions
+    private class ApfloatFunctions
         extends ApcomplexFunctions
     {
         @Override
@@ -699,6 +903,18 @@ public class ApfloatCalculatorImpl
             return ((Apfloat) x).frac();
         }
 
+        @Override
+        public Number inverseErf(Number x)
+        {
+            return ApfloatMath.inverseErf((Apfloat) x);
+        }
+
+        @Override
+        public Number inverseErfc(Number x)
+        {
+            return ApfloatMath.inverseErfc((Apfloat) x);
+        }
+
         @Override
         public Number max(Number x, Number y)
         {
@@ -814,7 +1030,7 @@ public class ApfloatCalculatorImpl
         }
     }
 
-    private static class AprationalFunctions
+    private class AprationalFunctions
         extends ApfloatFunctions
     {
         @Override
@@ -847,6 +1063,16 @@ public class ApfloatCalculatorImpl
             return ((Aprational) x).mod((Aprational) y);
         }
 
+        @Override
+        public Number pochhammer(Number x, Number y)
+        {
+            if (y instanceof Apint)
+            {
+                return AprationalMath.pochhammer((Aprational) x, (Apint) y);
+            }
+            return super.pochhammer(x, y);
+        }
+
         @Override
         protected Number pow(Number x, long y)
         {
@@ -912,7 +1138,7 @@ public class ApfloatCalculatorImpl
         }
     }
 
-    private static class ApintFunctions
+    private class ApintFunctions
         extends AprationalFunctions
     {
         @Override
@@ -1068,4 +1294,19 @@ public class ApfloatCalculatorImpl
         }
         return x;
     }
+
+    @Override
+    public void setInputPrecision(Long inputPrecision)
+    {
+        super.setInputPrecision(inputPrecision);
+        this.fixedPrecisionApcomplexHelper = new FixedPrecisionApcomplexHelper(inputPrecision == null ? Apfloat.INFINITE : inputPrecision);
+    }
+
+    private Number fixedOrArbitraryPrecision(java.util.function.Function<Apcomplex, Apcomplex> fixedPrecisionFunction, java.util.function.Function<Apcomplex, Apcomplex> arbitraryPrecisionFunction, Number x)
+    {
+        Apcomplex z = (Apcomplex) x;
+        return (getInputPrecision() != null && z.isZero() ? fixedPrecisionFunction.apply(z) : arbitraryPrecisionFunction.apply(z));
+    }
+
+    private FixedPrecisionApcomplexHelper fixedPrecisionApcomplexHelper = new FixedPrecisionApcomplexHelper(Apfloat.INFINITE);
 }
