@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -473,7 +473,6 @@ public class ApfloatContext
         {
             this.cleanupThread.setBuilderFactory(builderFactory);
         }
-
     }
 
     /**
@@ -1362,7 +1361,7 @@ public class ApfloatContext
             MemoryUsage memoryUsage = memoryBean.getHeapMemoryUsage();
             totalMemory = Math.max(memoryUsage.getCommitted(), memoryUsage.getMax());
         }
-        catch (NoClassDefFoundError ncdfe)
+        catch (NoClassDefFoundError | NullPointerException e)
         {
             // The ManagementFactory class might be unavailable
             totalMemory = Runtime.getRuntime().maxMemory();
