@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2024 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,10 +31,12 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.NoSuchElementException;
 
+import org.apfloat.spi.Util;
+
 /**
  * Various mathematical functions for arbitrary precision rational numbers.
  *
- * @version 1.11.0
+ * @version 1.13.0
  * @author Mikko Tommila
  */
 
@@ -369,6 +371,55 @@ public class AprationalMath
         return recursiveSum(x, 0, x.length - 1);
     }
 
+    /**
+     * Generates the first <code>n</code> terms in the continued fraction representation of <code>x</code>.<p>
+     *
+     * Note that the result length might be less than <code>n</code>, depending on the input value.
+     *
+     * @param x The number whose continued fraction terms should be generated.
+     * @param n The maximum number of terms to generate.
+     *
+     * @return The continued fraction.
+     *
+     * @exception IllegalArgumentException If <code>n</code> is less than one.
+     *
+     * @since 1.12.0
+     */
+
+    public static Apint[] continuedFraction(Aprational x, int n)
+    {
+        if (n <= 0)
+        {
+            throw new IllegalArgumentException("Maximum number of terms is not positive");
+        }
+        return Util.stream(ContinuedFractionHelper.continuedFraction(x)).limit(n).toArray(Apint[]::new);
+    }
+
+    /**
+     * Generates the first <code>n</code> convergents corresponding to the continued fraction of <code>x</code>.<p>
+     *
+     * Note that the result length might be less than <code>n</code>, depending on the input value.
+     *
+     * @param x The number whose continued fraction convergents should be generated.
+     * @param n The maximum number of convergents to generate.
+     *
+     * @return The convergents.
+     *
+     * @exception IllegalArgumentException If <code>n</code> is less than one.
+     *
+     * @since 1.12.0
+     */
+
+    public static Aprational[] convergents(Aprational x, int n)
+    {
+        if (n <= 0)
+        {
+            throw new IllegalArgumentException("Maximum number of convergents is not positive");
+        }
+        Iterator<Apint> continuedFraction = ContinuedFractionHelper.continuedFraction(x);
+        return Util.stream(ContinuedFractionHelper.convergents(continuedFraction, x.radix())).limit(n).toArray(Aprational[]::new);
+    }
+
     /**
      * Returns the greater of the two values.
      *
@@ -449,26 +500,46 @@ public class AprationalMath
         {
             return Apint.ZEROS[radix];
         }
-        Apint one = new Apint(1, radix);
-        return pochhammer(n.subtract(k).add(one), k.numerator()).divide(ApintMath.factorial(k.longValueExact(), radix));
+        Apint one = new Apint(1, radix),
+              f = ApintMath.factorial(ApfloatHelper.longValueExact(k.truncate()), radix);
+        return pochhammer(n.subtract(k).add(one), k.numerator()).divide(f);
     }
 
-    // Product of the numbers n * (n + 1) * (n + 2) * ... * (n + m - 1)
-    private static Aprational pochhammer(Aprational n, Apint m)
+    /**
+     * Pochhammer symbol.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * The asymptotic complexity is at least O(n<sup>2</sup>log&nbsp;n) and it is
+     * impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the pochhammer symbol.
+     *
+     * @param x The first argument.
+     * @param n The second argument.
+     *
+     * @return <code>(x)<sub>n</sub></code>
+     *
+     * @since 1.13.0
+     */
+
+    public static Aprational pochhammer(Aprational x, Apint n)
     {
-        assert (m.signum() >= 0);
-        Apint one = Apint.ONES[n.radix()];
-        if (m.signum() == 0)
+        Apint one = Apint.ONES[x.radix()];
+        if (n.signum() == 0)
         {
             return one;
         }
-        if (m.equals(one))
+        if (n.equals(one))
         {
-            return n;
+            return x;
         }
-        Apint two = new Apint(2, n.radix());
-        Apint k = m.divide(two);
-        return pochhammer(n, k).multiply(pochhammer(n.add(k), m.subtract(k)));
+        if (n.signum() < 0)
+        {
+            return one.divide(pochhammer(x.add(n), n.negate()));
+        }
+        Apint two = new Apint(2, x.radix());
+        Apint k = n.divide(two);
+        return pochhammer(x, k).multiply(pochhammer(x.add(k), n.subtract(k)));
     }
 
     /**
@@ -565,7 +636,7 @@ public class AprationalMath
         Apint one = Apint.ONES[radix],
               two = new Apint(2, radix);
         long p = Math.max(1, (long) Math.ceil(n * Math.log(n) / Math.log(radix))),
-             precision = ApfloatHelper.extendPrecision(Math.multiplyExact(Math.multiplyExact(2, n) + 1, p));  // 2 * n * p + 2 is not enough!
+             precision = ApfloatHelper.extendPrecision(Util.multiplyExact(Util.multiplyExact(2, n) + 1, p));  // 2 * n * p + 2 is not enough!
         Apint f2n1 = ApintMath.factorial(2 * n - 1,  radix);
         Apfloat z = ApfloatMath.scale(new Apfloat(1, precision, radix), -p),
                 v = ApfloatMath.scale(f2n1.multiply(ApfloatMath.tan(z)), -p);
@@ -711,7 +782,7 @@ public class AprationalMath
         Apint one = Apint.ONES[radix],
               two = new Apint(2, radix);
         long p = (long) Math.ceil(n * Math.log(n) / Math.log(radix)),
-             precision = ApfloatHelper.extendPrecision(Math.multiplyExact(Math.multiplyExact(2, n) + 1, p));  // 2 * n * p + 2 is not enough!
+             precision = ApfloatHelper.extendPrecision(Util.multiplyExact(Util.multiplyExact(2, n) + 1, p));  // 2 * n * p + 2 is not enough!
         Apint f2n1 = ApintMath.factorial(2 * n - 1,  radix);
         Apfloat z = ApfloatMath.scale(new Apfloat(1, precision, radix), -p);
 
