@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2024 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,9 +34,11 @@ import org.apfloat.spi.Util;
 import static java.util.Comparator.comparing;
 
 import static org.apfloat.ApcomplexMath.abs;
+import static org.apfloat.ApcomplexMath.exp;
 import static org.apfloat.ApcomplexMath.gamma;
 import static org.apfloat.ApcomplexMath.pow;
 import static org.apfloat.ApcomplexMath.sin;
+import static org.apfloat.ApcomplexMath.sqrt;
 import static org.apfloat.ApfloatMath.pi;
 import static org.apfloat.ApfloatMath.scale;
 
@@ -44,7 +46,7 @@ import static org.apfloat.ApfloatMath.scale;
  * Helper class for hypergeometric functions.
  *
  * @since 1.11.0
- * @version 1.11.0
+ * @version 1.13.0
  * @author Mikko Tommila
  */
 
@@ -70,6 +72,12 @@ class HypergeometricHelper
         private long precisionLoss;
     }
 
+    private static class NotConvergingException
+        extends RuntimeException
+    {
+            private static final long serialVersionUID = 1L;
+    }
+
     private class Hypergeometric2F1Helper
     {
         public Hypergeometric2F1Helper(boolean retry)
@@ -174,21 +182,6 @@ class HypergeometricHelper
             }
         }
 
-        private Apfloat offset(long scale)
-        {
-            Apfloat offset = scale(new Apfloat("0.1", workingPrecision, radix), scale);
-            return offset;
-        }
-
-        private Apfloat adjustOffset(Apfloat x, Apfloat offset)
-        {
-            if (x.scale() <= offset.scale())
-            {
-                return x;
-            }
-            return x.precision(Apfloat.INFINITE).add(offset).subtract(offset);
-        }
-
         public Apcomplex transform(Apcomplex s, Apcomplex c, Apcomplex base1, Apcomplex exp1, Apcomplex g1, Apcomplex g2, Apcomplex a1, Apcomplex b1, Apcomplex c1, Apcomplex base2, Apcomplex exp2, Apcomplex base3, Apcomplex exp3, Apcomplex g3, Apcomplex g4, Apcomplex a2, Apcomplex b2, Apcomplex c2, Apcomplex z)
         {
             Apcomplex term1,
@@ -210,7 +203,7 @@ class HypergeometricHelper
                 term2 = pow(base2, exp2).multiply(pow(base3, exp3)).divide(gamma(g3).multiply(gamma(g4)).multiply(gamma(c2))).multiply(evaluate(a2, b2, c2, z));
             }
             Apcomplex d = term1.subtract(term2);
-            long precisionLoss = (d.real().signum() == 0 && d.imag().signum() == 0 ? workingPrecision : targetPrecision - d.precision());
+            long precisionLoss = (d.isZero() ? workingPrecision : targetPrecision - d.precision());
             if (retry && precisionLoss > 1) // Allow a precision loss of 1 (which happens often), otherwise retry with increased precision
             {
                 throw new RetryException(precisionLoss);
@@ -294,7 +287,7 @@ class HypergeometricHelper
             @Override
             public boolean isApplicable(Apcomplex z)
             {
-                return z.real().signum() != 0 || z.imag().signum() != 0;
+                return !z.isZero();
             }
 
             @Override
@@ -372,7 +365,7 @@ class HypergeometricHelper
             @Override
             public boolean isApplicable(Apcomplex z)
             {
-                return z.real().signum() != 0 || z.imag().signum() != 0;
+                return !z.isZero();
             }
 
             @Override
@@ -449,7 +442,104 @@ class HypergeometricHelper
         return new HypergeometricHelper(a, b, z).hypergeometricPFQ();
     }
 
-    // In the general case (p = q + 1) this works only for |z| < 1
+    /**
+     * Regularized generalized hypergeometric function <i><sub>p</sub>F̃<sub>q</sub></i>.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * It is impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the function.
+     *
+     * @param a The first argument.
+     * @param b The second argument.
+     * @param z The third argument.
+     *
+     * @return <i><sub>p</sub>F̃<sub>q</sub>(a<sub>1</sub>, …, a<sub>p</sub>; b<sub>1</sub>, …, b<sub>q</sub>; z)</i>
+     *
+     * @throws ArithmeticException If the series does not converge.
+     *
+     * @since 1.13.0
+     */
+
+    public static Apcomplex hypergeometricPFQRegularized(Apcomplex[] a, Apcomplex[] b, Apcomplex z)
+    {
+        return new HypergeometricHelper(a, b, z).hypergeometricPFQRegularized();
+    }
+
+    private Apcomplex hypergeometricPFQRegularized()
+    {
+        Apcomplex result;
+        Apfloat n = null;
+        int j = -1;
+        // Find smallest integer from b that is <= 0, if any
+        for (int i = 0; i < b.length; i++)
+        {
+            Apfloat br = b[i].real().negate();
+            if (b[i].isInteger() && br.signum() >= 0)
+            {
+                if (n == null)
+                {
+                    j = i;
+                    n = br;
+                }
+                else if (br.compareTo(n) > 0)
+                {
+                    j = i;
+                    n = br;
+                }
+            }
+        }
+        if (n == null)
+        {
+            // None of the b is a nonpositive integer, regularization is trivial
+            Apcomplex[] gamma = Arrays.stream(b).map(this::ensureGammaPrecision).map(ApcomplexMath::gamma).toArray(Apcomplex[]::new);
+            result = hypergeometricPFQ().divide(ApcomplexMath.product(gamma));
+        }
+        else
+        {
+            // At least one of the b is a nonpositive integer, regularization needs to be done by omitting the terms where the divisor is infinite
+            int radix = z.radix();
+            Apfloat n1 = n.add(Apint.ONES[radix]);
+            Apcomplex[] pochhammer = Arrays.stream(a).map(ai -> ApcomplexMath.pochhammer(ai, n1)).toArray(Apcomplex[]::new);
+            result = ApcomplexMath.product(pochhammer);
+            if (!result.isZero())
+            {
+                Apfloat n2 = n.add(new Apint(2, radix));
+                Apcomplex[] gamma = Arrays.stream(b).map(n1::add).map(this::ensureGammaPrecision).map(ApcomplexMath::gamma).toArray(Apcomplex[]::new);
+                result = result.multiply(ApcomplexMath.pow(z, n1)).divide(ApcomplexMath.gamma(n2).multiply(ApcomplexMath.product(gamma)));
+                a = Arrays.stream(a).map(n1::add).toArray(Apcomplex[]::new);
+                b = Arrays.stream(b).map(n1::add).toArray(Apcomplex[]::new);
+                b[j] = n2;
+                result = result.multiply(hypergeometricPFQ());
+            }
+        }
+        return result;
+    }
+
+    /**
+     * Tricomi's confluent hypergeometric function <i>U</i>.
+     * Also known as the confluent hypergeometric function of the second kind.<p>
+     *
+     * @implNote
+     * This implementation is <i>slow</i>, meaning that it isn't a <i>fast algorithm</i>.
+     * It is impractically slow beyond a precision of a few thousand digits. At the time of
+     * implementation no generic fast algorithm is known for the function.
+     *
+     * @param a The first argument.
+     * @param b The second argument.
+     * @param z The third argument.
+     *
+     * @return <i>U(a, b, z)</i>
+     *
+     * @since 1.13.0
+     */
+
+    public static Apcomplex hypergeometricU(Apcomplex a, Apcomplex b, Apcomplex z)
+    {
+        return new HypergeometricHelper(new Apcomplex[] { a }, new Apcomplex[] { b }, z).hypergeometricU();
+    }
+
+    // In the general case (p = q + 1) this works only for |z| < 1, and in general, may converge slowly for very large |z|
     private Apcomplex hypergeometricPFQ()
         throws ArithmeticException, ApfloatRuntimeException
     {
@@ -468,7 +558,7 @@ class HypergeometricHelper
         }
         if (a.length > b.length + 1L)
         {
-            if (z.real().signum() == 0 && z.imag().signum() == 0)
+            if (z.isZero())
             {
                 return new Apfloat(1, targetPrecision, radix);
             }
@@ -482,12 +572,160 @@ class HypergeometricHelper
         {
             return ApcomplexMath.pow(one.subtract(z), a[0].negate());
         }
+        if (a.length == 0 && b.length == 1)
+        {
+            return hypergeometric0F1(b[0], z);
+        }
+        if (a.length == 1 && b.length == 1)
+        {
+            return hypergeometric1F1(a[0], b[0], z);
+        }
         assert (b.length > 0);
 
         result = evaluate(a, b, z);
         return ApfloatHelper.limitPrecision(result, targetPrecision);
     }
 
+    private Apcomplex hypergeometric0F1(Apcomplex b, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        if (abs(z).doubleValue() > targetPrecision * Math.log(radix))
+        {
+            // Evaluate with 1F1
+            Apcomplex sqrtZ = sqrt(z);
+            Apint two = new Apint(2, radix),
+                  four = new Apint(4, radix);
+            Aprational half = new Aprational(one, two);
+            Apcomplex result = exp(two.negate().multiply(sqrtZ)).multiply(hypergeometric1F1(b.subtract(half), two.multiply(b).subtract(one), four.multiply(sqrtZ)));
+            return ApfloatHelper.limitPrecision(result, targetPrecision);
+        }
+
+        Apcomplex result = evaluate(new Apcomplex[0], new Apcomplex[] { b }, z);
+        return ApfloatHelper.limitPrecision(result, targetPrecision);
+    }
+
+    private Apcomplex hypergeometric1F1(Apcomplex a, Apcomplex b, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        if (abs(z).doubleValue() > targetPrecision * Math.log(radix))
+        {
+            try
+            {
+                // Evaluate with U*
+                Apcomplex ba = ensurePrecision(b.subtract(a));
+                Apcomplex result = zero;
+                if (!ba.isInteger() || ba.real().signum() > 0)
+                {
+                    result = pow(z.negate(), a.negate()).divide(gamma(ensureGammaPrecision(ba))).multiply(hypergeometricUStar(a, b, z));
+                }
+                result = result.add(pow(z, ba.negate()).multiply(exp(z)).divide(gamma(ensureGammaPrecision(a))).multiply(hypergeometricUStar(ba, b, z.negate()))).multiply(gamma(ensureGammaPrecision(b)));
+                return ApfloatHelper.limitPrecision(result, targetPrecision);
+            } catch (NotConvergingException nce)
+            {
+                // Ignore and retry with the (possibly very slow) direct evaluation of the series
+            }
+        }
+
+        Apcomplex result = hypergeometric1F1series(a, b, z);
+        return ApfloatHelper.limitPrecision(result, targetPrecision);
+    }
+
+    private Apcomplex hypergeometric1F1series(Apcomplex a, Apcomplex b, Apcomplex z)
+    {
+        Apcomplex factor = one;
+        if (z.real().signum() < 0)
+        {
+            // Kummer transformation
+            factor = ApcomplexMath.exp(z);
+            a = ensurePrecision(b.subtract(a));
+            z = z.negate();
+        }
+
+        Apcomplex result = evaluate(new Apcomplex[] { a }, new Apcomplex[] { b }, z).multiply(factor);
+        return result;
+    }
+
+    private Apcomplex hypergeometricUStar(Apcomplex a, Apcomplex b, Apcomplex z)
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        Apcomplex ab1 = ensurePrecision(a.subtract(b).add(one));
+        Apcomplex result = evaluate(new Apcomplex[] { a, ab1 }, new Apcomplex[0], one.divide(z).negate());
+        return result;
+    }
+
+    private Apcomplex hypergeometricU()
+        throws ArithmeticException, ApfloatRuntimeException
+    {
+        if (abs(z).doubleValue() > targetPrecision * Math.log(radix))
+        {
+            try
+            {
+                // Evaluate with U*
+                Apcomplex result = pow(z, a[0].negate()).multiply(hypergeometricUStar(a[0], b[0], z));
+                return ApfloatHelper.limitPrecision(result, targetPrecision);
+            } catch (NotConvergingException nce)
+            {
+                // Ignore and retry with the (possibly very slow) direct evaluation of the series
+            }
+        }
+
+        // Evaluate two 1F1 functions, this often results in cancellation of significant digits so we may retry with higher precision
+        Apcomplex result;
+        long precisionLoss;
+        do
+        {
+            Apcomplex a = this.a[0],
+                      b = this.b[0];
+            a = ensurePrecision(a);
+            b = ensurePrecision(b);
+            z = ensurePrecision(z);
+
+            // First check if b is an integer or near-integer and adjust if necessary
+            if (b.isInteger())
+            {
+                long digitLoss = workingPrecision;
+                workingPrecision = Util.ifFinite(workingPrecision, workingPrecision + digitLoss);
+                Apfloat offset = offset(-digitLoss);
+                b = new Apcomplex(b.real().precision(Apfloat.INFINITE).add(offset), b.imag());
+                a = ensurePrecision(a);
+                b = ensurePrecision(b);
+                z = ensurePrecision(z);
+            }
+            else
+            {
+                Apint bRounded = RoundingHelper.roundToInteger(b.real(), RoundingMode.HALF_EVEN).truncate();
+                long digitLoss = -b.subtract(bRounded).scale();
+                if (digitLoss > 0)
+                {
+                    workingPrecision = Util.ifFinite(workingPrecision, workingPrecision + digitLoss);
+                    a = ensurePrecision(a);
+                    b = ensurePrecision(b);
+                    z = ensurePrecision(z);
+                }
+            }
+
+            Apcomplex ab1 = ensureGammaPrecision(a.subtract(b).add(one));
+            result = zero;
+            if (!ab1.isInteger() || ab1.real().signum() > 0)
+            {
+                Apcomplex b1n = ensureGammaPrecision(one.subtract(b));
+                result = gamma(b1n).divide(gamma(ab1)).multiply(hypergeometric1F1series(a, b, z));
+            }
+            if (!a.isInteger() || a.real().signum() > 0)
+            {
+                Apint two = new Apint(2, radix);
+                Apcomplex b1 = ensureGammaPrecision(b.subtract(one)),
+                          b1n = ensurePrecision(one.subtract(b)),
+                          b2 = ensurePrecision(two.subtract(b));
+                a = ensureGammaPrecision(a);
+                result = result.add(gamma(b1).divide(gamma(a)).multiply(pow(z, b1n)).multiply(hypergeometric1F1series(ab1, b2, z)));
+            }
+            precisionLoss = targetPrecision - result.precision();
+            workingPrecision = Util.ifFinite(workingPrecision, workingPrecision + precisionLoss);
+        } while (precisionLoss > 0);
+        return ApfloatHelper.limitPrecision(result, targetPrecision);
+    }
+
     private Apcomplex hypergeometric2F1(Apcomplex a, Apcomplex b, Apcomplex c, Apcomplex z)
         throws ArithmeticException, ApfloatRuntimeException
     {
@@ -515,9 +753,10 @@ class HypergeometricHelper
                 return zero;
             }
             Apcomplex cab = s.subtract(b);
-            s = ApfloatHelper.ensurePrecision(s, workingPrecision);
-            t = ApfloatHelper.ensurePrecision(t, workingPrecision);
-            cab = ApfloatHelper.ensurePrecision(cab, workingPrecision);
+            s = ApfloatHelper.ensureGammaPrecision(s, workingPrecision);
+            t = ApfloatHelper.ensureGammaPrecision(t, workingPrecision);
+            cab = ApfloatHelper.ensureGammaPrecision(cab, workingPrecision);
+            c = ApfloatHelper.ensureGammaPrecision(c, workingPrecision);
             return ApcomplexMath.gamma(c).multiply(ApcomplexMath.gamma(cab)).divide(ApcomplexMath.gamma(s).multiply(ApcomplexMath.gamma(t)));
         }
         Apcomplex zDoublePrecision = z.precision(ApfloatHelper.getDoublePrecision(radix));
@@ -558,13 +797,13 @@ class HypergeometricHelper
 
     private Apcomplex checkResult()
     {
-        Apfloat minNonPositiveIntegerA = minNonPositiveInteger(a),
-                minNonPositiveIntegerB = minNonPositiveInteger(b);
+        Apfloat minNonPositiveIntegerA = maxNonPositiveInteger(a),
+                minNonPositiveIntegerB = maxNonPositiveInteger(b);
         if (minNonPositiveIntegerB != null && (minNonPositiveIntegerA == null || minNonPositiveIntegerA.compareTo(minNonPositiveIntegerB) < 0))
         {
             throw new ArithmeticException("Division by zero");
         }
-        if (z.real().signum() == 0 && z.imag().signum() == 0)
+        if (z.isZero())
         {
             return new Apfloat(1, targetPrecision, radix);
         }
@@ -581,9 +820,9 @@ class HypergeometricHelper
         return null;
     }
 
-    public static Apfloat minNonPositiveInteger(Apcomplex... a)
+    public static Apfloat maxNonPositiveInteger(Apcomplex... a)
     {
-        return Arrays.stream(a).filter(Apcomplex::isInteger).map(Apcomplex::real).filter(x -> x.signum() <= 0).reduce(ApfloatMath::min).orElse(null);
+        return Arrays.stream(a).filter(Apcomplex::isInteger).map(Apcomplex::real).filter(x -> x.signum() <= 0).reduce(ApfloatMath::max).orElse(null);
     }
 
     // It could be that the terms initially grow for a long time, until they start getting smaller
@@ -597,6 +836,7 @@ class HypergeometricHelper
         long precisionLoss = 0,
              extraPrecision,
              extendedPrecision = ApfloatHelper.extendPrecision(workingPrecision, minN.scale()); // Estimate for accumulated round-off error precision due to repeated multiplication only (not scale based digit loss, see below)
+        boolean divergentSeries = a.length - b.length > 1;
 
         ensurePrecision(a, a, extendedPrecision);
         ensurePrecision(b, b, extendedPrecision);
@@ -608,7 +848,8 @@ class HypergeometricHelper
             Apint i = zero;
             Apcomplex numerator = one,
                       denominator = one,
-                      t;
+                      o,
+                      t = null;
 
             extraPrecision = precisionLoss;
             s = one;
@@ -621,9 +862,9 @@ class HypergeometricHelper
                     numerator = numerator.multiply(a[j]);
                     a[j] = a[j].add(one);
                 }
-                if (numerator.real().signum() == 0 && numerator.imag().signum() == 0)
+                if (numerator.isZero())
                 {
-                    break;  // It was a polynomial
+                    return s;   // It was a polynomial
                 }
                 numerator = numerator.multiply(z);
                 for (int j = 0; j < b.length; j++)
@@ -632,12 +873,13 @@ class HypergeometricHelper
                     b[j] = b[j].add(one);
                 }
                 denominator = denominator.multiply(i);
+                o = t;
                 t = numerator.divide(denominator);
                 s = s.add(t);
                 maxSScale = Math.max(maxSScale, s.scale());
-            } while (i.compareTo(minN) <= 0 || s.real().signum() == 0 && s.imag().signum() == 0 || s.scale() - t.scale() <= workingPrecision); // Subtraction might overflow
+            } while (i.compareTo(minN) <= 0 || divergentSeries && checkDivergence(o, t) || s.isZero() || s.scale() - t.scale() <= workingPrecision);  // Subtraction might overflow
 
-            precisionLoss = (s.real().signum() == 0 && s.imag().signum() == 0 ? extendedPrecision : maxSScale - s.scale()); // Loss due to scale of s reduced from its peak (loss off most significant digits)
+            precisionLoss = (s.isZero() ? extendedPrecision : maxSScale - s.scale()); // Loss due to scale of s reduced from its peak (loss off most significant digits)
             if (workingPrecision - s.precision() > 1)  // Often the precision is reduced by 1
             {
                 precisionLoss = Util.ifFinite(precisionLoss, precisionLoss + workingPrecision - s.precision()); // Loss due to accumulation (loss off least significant digits)
@@ -653,6 +895,16 @@ class HypergeometricHelper
         return s;
     }
 
+    private boolean checkDivergence(Apcomplex old, Apcomplex term)
+    {
+        // Check if the divergent series reached the smallest term already, even though the required precision was not reached
+        if (old != null && abs(old).compareTo(abs(term)) < 0)
+        {
+            throw new NotConvergingException();
+        }
+        return false;
+    }
+
     // See: https://def.fe.up.pt/pipermail/maxima-discuss/2006.txt "Methods for numerically difficult cases of 2F1(a,b;c|z)", alternative algorithm by Bill Gosper
     private Apcomplex alternative(Apcomplex a, Apcomplex b, Apcomplex c, Apcomplex z)
     {
@@ -702,6 +954,21 @@ class HypergeometricHelper
         return f;
     }
 
+    private Apfloat offset(long scale)
+    {
+        Apfloat offset = scale(new Apfloat("0.1", workingPrecision, radix), scale);
+        return offset;
+    }
+
+    private Apfloat adjustOffset(Apfloat x, Apfloat offset)
+    {
+        if (x.scale() <= offset.scale())
+        {
+            return x;
+        }
+        return x.precision(Apfloat.INFINITE).add(offset).subtract(offset);
+    }
+
     private Apcomplex ensurePrecision(Apcomplex z)
     {
         return ApfloatHelper.ensurePrecision(z, workingPrecision);
@@ -715,6 +982,11 @@ class HypergeometricHelper
         }
     }
 
+    private Apcomplex ensureGammaPrecision(Apcomplex z)
+    {
+        return ApfloatHelper.ensureGammaPrecision(z, workingPrecision);
+    }
+
     private long targetPrecision,
                  workingPrecision;
     private int radix;
