@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -131,7 +131,7 @@ public class RawtypeBuilderFactory
     @Override
     public int getElementSize()
     {
-        return sizeof(rawtype);
+        return RawType.BYTES;
     }
 
     @Override
