@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2021 Mikko Tommila
+ * Copyright (c) 2002-2023 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -99,7 +99,7 @@ public class RawtypeMemoryDataStorage
         ApfloatContext ctx = ApfloatContext.getContext();
         int readSize = (int) Math.min(size, dataStorage.getSize()),
             position = 0,
-            bufferSize = ctx.getBlockSize() / sizeof(rawtype);
+            bufferSize = ctx.getBlockSize() / RawType.BYTES;
 
         while (readSize > 0)
         {
