@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2002-2023 Mikko Tommila
+ * Copyright (c) 2002-2024 Mikko Tommila
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,7 @@ import java.util.Locale;
 import junit.framework.TestSuite;
 
 /**
- * @version 1.10.1
+ * @version 1.13.0
  * @author Mikko Tommila
  */
 
@@ -59,6 +59,7 @@ public class ApcomplexTest
     {
         TestSuite suite = new TestSuite();
 
+        suite.addTest(new ApcomplexTest("testComparator"));
         suite.addTest(new ApcomplexTest("testConstructor"));
         suite.addTest(new ApcomplexTest("testStringConstructor"));
         suite.addTest(new ApcomplexTest("testStreamConstructor"));
@@ -67,6 +68,7 @@ public class ApcomplexTest
         suite.addTest(new ApcomplexTest("testPrecision"));
         suite.addTest(new ApcomplexTest("testScale"));
         suite.addTest(new ApcomplexTest("testSize"));
+        suite.addTest(new ApcomplexTest("testIsZero"));
         suite.addTest(new ApcomplexTest("testIsInteger"));
         suite.addTest(new ApcomplexTest("testNegate"));
         suite.addTest(new ApcomplexTest("testAdd"));
@@ -87,6 +89,29 @@ public class ApcomplexTest
         return suite;
     }
 
+    public static void testComparator()
+    {
+        assertEquals("REAL_IMAG 1 compare 2", -1, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("1"), new Apcomplex("2")));
+        assertEquals("REAL_IMAG 2 compare 2", 0, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("2"), new Apcomplex("2")));
+        assertEquals("REAL_IMAG 2 compare 1", 1, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("2"), new Apcomplex("1")));
+        assertEquals("REAL_IMAG (1, 2) compare (2, 1)", -1, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("(1,2)"), new Apcomplex("(2,1)")));
+        assertEquals("REAL_IMAG (2, 1) compare (2, 2)", -1, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("(2,1)"), new Apcomplex("(2,2)")));
+        assertEquals("REAL_IMAG (2, 2) compare (2, 2)", 0, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("(2,2)"), new Apcomplex("(2,2)")));
+        assertEquals("REAL_IMAG (2, 2) compare (2, 1)", 1, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("(2,2)"), new Apcomplex("(2,1)")));
+        assertEquals("REAL_IMAG (2, -2) compare (2, 1)", -1, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("(2,-2)"), new Apcomplex("(2,1)")));
+        assertEquals("REAL_IMAG (2, -2) compare (2, 2)", -1, Apcomplex.REAL_IMAG_ORDER.compare(new Apcomplex("(2,-2)"), new Apcomplex("(2,2)")));
+
+        assertEquals("REAL_ABS_IMAG 1 compare 2", -1, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("1"), new Apcomplex("2")));
+        assertEquals("REAL_ABS_IMAG 2 compare 2", 0, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("2"), new Apcomplex("2")));
+        assertEquals("REAL_ABS_IMAG 2 compare 1", 1, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("2"), new Apcomplex("1")));
+        assertEquals("REAL_ABS_IMAG (1, 2) compare (2, 1)", -1, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("(1,2)"), new Apcomplex("(2,1)")));
+        assertEquals("REAL_ABS_IMAG (2, 1) compare (2, 2)", -1, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("(2,1)"), new Apcomplex("(2,2)")));
+        assertEquals("REAL_ABS_IMAG (2, 2) compare (2, 2)", 0, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("(2,2)"), new Apcomplex("(2,2)")));
+        assertEquals("REAL_ABS_IMAG (2, 2) compare (2, 1)", 1, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("(2,2)"), new Apcomplex("(2,1)")));
+        assertEquals("REAL_ABS_IMAG (2, -2) compare (2, 1)", 1, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("(2,-2)"), new Apcomplex("(2,1)")));
+        assertEquals("REAL_ABS_IMAG (2, -2) compare (2, 2)", -1, Apcomplex.REAL_ABS_IMAG_ORDER.compare(new Apcomplex("(2,-2)"), new Apcomplex("(2,2)")));
+    }
+
     public static void testConstructor()
     {
         Apcomplex a = new Apcomplex(new Apfloat(5));
@@ -423,6 +448,18 @@ public class ApcomplexTest
         assertEquals("0 size", 0, a.size());
     }
 
+    public static void testIsZero()
+    {
+        Apcomplex a = new Apcomplex("(1, 1)");
+        assertFalse("(1, 1)", a.isZero());
+        a = new Apcomplex("(1.5, 0)");
+        assertFalse("(1.5, 0)", a.isZero());
+        a = new Apcomplex("(0, 1)");
+        assertFalse("(0, 1)", a.isZero());
+        a = new Apcomplex("0");
+        assertTrue("0", a.isZero());
+    }
+
     public static void testIsInteger()
     {
         Apcomplex a = new Apcomplex("(1, 1)");
@@ -527,17 +564,57 @@ public class ApcomplexTest
 
     public static void testNumberValuesExact()
     {
-        Apcomplex a = new Apcomplex("5.5");
-        assertEquals("5.5 longValueExact", 5, a.longValueExact());
-        assertEquals("5.5 intValueExact", 5, a.intValueExact());
-        assertEquals("5.5 shortValueExact", 5, a.shortValueExact());
-        assertEquals("5.5 byteValueExact", 5, a.byteValueExact());
+        Apcomplex a = new Apcomplex("5");
+        assertEquals("5 longValueExact", 5, a.longValueExact());
+        assertEquals("5 intValueExact", 5, a.intValueExact());
+        assertEquals("5 shortValueExact", 5, a.shortValueExact());
+        assertEquals("5 byteValueExact", 5, a.byteValueExact());
 
-        a = new Apcomplex("-5.5");
-        assertEquals("-5.5 longValueExact", -5, a.longValueExact());
-        assertEquals("-5.5 intValueExact", -5, a.intValueExact());
-        assertEquals("-5.5 shortValueExact", -5, a.shortValueExact());
-        assertEquals("-5.5 byteValueExact", -5, a.byteValueExact());
+        a = new Apcomplex("-5");
+        assertEquals("-5 longValueExact", -5, a.longValueExact());
+        assertEquals("-5 intValueExact", -5, a.intValueExact());
+        assertEquals("-5 shortValueExact", -5, a.shortValueExact());
+        assertEquals("-5 byteValueExact", -5, a.byteValueExact());
+
+        try
+        {
+            new Apcomplex("5.5").longValueExact();
+            fail("Non-integer number accepted");
+        }
+        catch (ArithmeticException ae)
+        {
+            // OK
+        }
+
+        try
+        {
+            new Apcomplex("5.5").intValueExact();
+            fail("Non-integer number accepted");
+        }
+        catch (ArithmeticException ae)
+        {
+            // OK
+        }
+
+        try
+        {
+            new Apcomplex("5.5").shortValueExact();
+            fail("Non-integer number accepted");
+        }
+        catch (ArithmeticException ae)
+        {
+            // OK
+        }
+
+        try
+        {
+            new Apcomplex("5.5").byteValueExact();
+            fail("Non-integer number accepted");
+        }
+        catch (ArithmeticException ae)
+        {
+            // OK
+        }
 
         try
         {
